SprayAddon = {};

source("dataS/scripts/vehicles/specializations/SprayerAreaEvent.lua");

function SprayAddon.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(Attachable, specializations);
end;

function SprayAddon:load(xmlFile)
	self.fertilize = false;
	self.killWeeds = false;
	self.sprayTank = nil;
	self.lastSprayingArea = 0;
	self.fertUsage = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.fertUsage"), 9);
	self.HerbUsage = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.HerbUsage"), 0.5);
	-- Hud
	self.fertxPos = Utils.getNoNil(getXMLFloat(xmlFile,  "vehicle.ferthudPos#posX"), 0.6);
	self.fertyPos = Utils.getNoNil(getXMLFloat(xmlFile,  "vehicle.ferthudPos#posY"), 0.02);
	self.HerbxPos = Utils.getNoNil(getXMLFloat(xmlFile,  "vehicle.HerbhudPos#posX"), 0.6);
	self.HerbyPos = Utils.getNoNil(getXMLFloat(xmlFile,  "vehicle.HerbhudPos#posY"), 0.10);
	self.xPos = Utils.getNoNil(getXMLFloat(xmlFile,  "vehicle.hudPos#posX"), 0.6);
	self.yPos = Utils.getNoNil(getXMLFloat(xmlFile,  "vehicle.hudPos#posY"), 0.06);
end;

function SprayAddon:delete()
end;

function SprayAddon:mouseEvent(posX, posY, isDown, isUp, button)
end;

function SprayAddon:keyEvent(unicode, sym, modifier, isDown)
end;

function SprayAddon:update(dt)	
end;

function SprayAddon:updateTick(dt)	
	if self.attacherVehicle ~= nil and self.isTurnedOn then
		if self.sprayTank ~= nil then
			self.fertilize = true;
		else
			self.fertilize = false;
		end;
		if self.attacherVehicle.isHerbicide then
			self.killWeeds = true;
		else		
			self.killWeeds = false;
		end;
	else
		self.fertilize = false;
		self.killWeeds = false;
	end;
	if self.attacherVehicle ~= nil then
		local isHerbicide = Utils.getNoNil(self.attacherVehicle.isHerbicide,false) and g_currentMission.fruits[FruitUtil.FRUITTYPE_WEED] ~= nil;
		if self.fertilize or self.killWeeds then		
			local cuttingAreasSend = {};
			for _,cuttingArea in pairs(self.cuttingAreas) do
				if self:getIsAreaActive(cuttingArea) then
					local x,_,z = getWorldTranslation(cuttingArea.start);
					if g_currentMission:getIsFieldOwnedAtWorldPos(x,z) then
						local x1,_,z1 = getWorldTranslation(cuttingArea.width);
						local x2,_,z2 = getWorldTranslation(cuttingArea.height);	
						if self.fertilize then
							local sqm = math.abs((z1-z)*(x2-x) - (x1-x)*(z2-z)); -- this is the cross product with y=0
							self.lastSprayingArea = self.lastSprayingArea + sqm;	
							table.insert(cuttingAreasSend, {x,z,x1,z1,x2,z2});
						end;
						if self.killWeeds then
							local x,z, widthX,widthZ, heightX,heightZ = Utils.getXZWidthAndHeight(g_currentMission.fruits[FruitUtil.FRUITTYPE_WEED].id, x, z, x1, z1, x2, z2);
							setDensityParallelogram(g_currentMission.fruits[FruitUtil.FRUITTYPE_WEED].id, x, z, widthX, widthZ, heightX, heightZ, 1);
						end;
					else
						showFieldNotOwnedWarning = true;
					end
				end;
			end;
			if (table.getn(cuttingAreasSend) > 0) and self.fertilize then
				SprayerAreaEvent.runLocally(cuttingAreasSend);
				g_server:broadcastEvent(SprayerAreaEvent:new(cuttingAreasSend));
			end;
		end;
		if self.fertilize then
			if self.sprayTank.fillLevel > 0 then
				local usage = self.fertUsage
				local difLvl = g_currentMission.missionStats.difficulty;
				local difMult = math.max(math.pow(difLvl,difLvl-1) +4 * (math.pow(difLvl-3,difLvl)),1);
				self.sprayTank:setFillLevel(self.sprayTank.fillLevel - (self.attacherVehicle.lastSpeed * usage * difMult) , self.sprayTank.currentFillType); --4.5         
			else
				if self.attacherVehicle.isSprayerTank and self.attacherVehicle.fillLevel > 0 and not self.attacherVehicle.isHerbicide then
					local usage = self.fertUsage
					local difLvl = g_currentMission.missionStats.difficulty;
					local difMult = math.max(math.pow(difLvl,difLvl-1) +4 * (math.pow(difLvl-3,difLvl)),1);
					self.attacherVehicle:setFillLevel(self.attacherVehicle.fillLevel - (self.attacherVehicle.lastSpeed * usage * difMult) , self.attacherVehicle.currentFillType);
				else
					self.fertilize = false;
					if self.attacherVehicle.isAITractorActivated then		
						AITractor.stopAITractor(self.attacherVehicle);
					end;
				end;
			end;
		end;
		if self.killWeeds then
			if self.attacherVehicle.fillLevel > 0 then
				local usage = self.HerbUsage
				local difLvl = g_currentMission.missionStats.difficulty;
				local difMult = math.max(math.pow(difLvl,difLvl-1) +4 * (math.pow(difLvl-3,difLvl)),1);
				self.attacherVehicle:setFillLevel(self.attacherVehicle.fillLevel - (self.attacherVehicle.lastSpeed * usage * difMult) , self.attacherVehicle.currentFillType); --4.5         
			else
				self.killWeeds = false;
				if self.attacherVehicle.isAITractorActivated then		
					AITractor.stopAITractor(self.attacherVehicle);
				end;
			end;
		end;
	end;
end;

function SprayAddon:attachImplement(implement)
    if implement.object.isSprayerTank then
		self.sprayTank = implement.object;
	end;
end;

function SprayAddon:detachImplement(implementIndex)
	local object = self.attachedImplements[implementIndex].object;
    if object.attacherJoint.jointType == Vehicle.JOINTTYPE_TRAILER then
        self.fertilize = false;
        self.sprayTank = nil;
    end;
end;

function SprayAddon:draw()
	--Herbicide draw script
	if self.attacherVehicle ~= nil then
		if self.attacherVehicle.isSprayerTank then
			if self.attacherVehicle.isHerbicide then				
				local percent2 = 0;
				local level2 = self.attacherVehicle.fillLevel;
				local capacity2 = self.attacherVehicle.capacity;
				percent2 = level2 / capacity2 * 100;
				setTextBold(true);
				setTextAlignment(RenderText.ALIGN_CENTER);
				
				if percent2 > 15 then
					setTextColor(0, 1, 0, 1);
				elseif percent2 > 5 and percent2 < 15 then
					setTextColor(1, 1, 0, 1);
				else
					setTextColor(1, 0, 0, 1);
				end;
		
				renderText(self.HerbxPos + 0.075, self.HerbyPos + 0.013, 0.02, string.format("Herbicide %d (%d%%)", level2, percent2));
			else
				local percent2 = 0;
				local level2 = self.attacherVehicle.fillLevel;
				local capacity2 = self.attacherVehicle.capacity;
				percent2 = level2 / capacity2 * 100;
				setTextBold(true);
				setTextAlignment(RenderText.ALIGN_CENTER);
		
				if percent2 > 15 then
					setTextColor(0, 1, 0, 1);
				elseif percent2 > 5 and percent2 < 15 then
					setTextColor(1, 1, 0, 1);
				else
					setTextColor(1, 0, 0, 1);
				end;
		
				renderText(self.HerbxPos + 0.075, self.HerbyPos + 0.013, 0.02, string.format("Fertilizer %d (%d%%)", level2, percent2));
			end;
		end;
	end;
	--Herbicide draw script
	
	--Fertilizer draw script
	if self.sprayTank ~= nil then
		
		local percent = 0;
		local percent1 = 0;
		local level = self.sprayTank.fillLevel;
		local capacity = self.sprayTank.capacity;
		local level1 = self.fillLevel;
		local capacity1 = self.capacity;
		percent = level / capacity * 100;
		percent1 = level1 / capacity1 * 100;

		-- write fertilizer and seed level
		setTextBold(true);
		setTextAlignment(RenderText.ALIGN_CENTER);
	
		if percent > 15 then
			setTextColor(0, 1, 0, 1);
		elseif percent > 5 and percent < 15 then
			setTextColor(1, 1, 0, 1);
		else
			setTextColor(1, 0, 0, 1);
		end;
	
		renderText(self.fertxPos + 0.075, self.fertyPos + 0.013, 0.02, string.format("Fertilizer %d (%d%%)", level, percent));
	
		if percent1 > 15 then
			setTextColor(0, 1, 0, 1);
		elseif percent1 > 5 and percent1 < 15 then
			setTextColor(1, 1, 0, 1);
		else
			setTextColor(1, 0, 0, 1);
		end;
	
		renderText(self.xPos + 0.075, self.yPos + 0.013, 0.02, string.format("Seeds %d (%d%%)", level1, percent1));
	end;
	--Fertilizer draw script
end;